from __future__ import annotations

from prettyqt import widgets


class GraphicsPathItem(widgets.AbstractGraphicsShapeItemMixin, widgets.QGraphicsPathItem):
    """Path item that you can add to a QGraphicsScene."""
