"""All editors adhere to a get/set_value interface + value_changed signal.

Additional "core" editors:
- LineEdit
- DoubleSlider
- Slider
- FontComboBox
- ComboBox
- CheckBox
- PushButton
- CalendarWidget
- DateEdit
- DateTimeEdit
- TimeEdit
- Dial
- SpinBox
- DoubleSpinBox
- KeySequenceEdit
- LCDNumber
- ListWidget
- PlainTextEdit
- TextEdit
- ProgressBar
- TextBrowser
"""

from __future__ import annotations
