from __future__ import annotations

from prettyqt.qt import QtCore


class RandomGenerator(QtCore.QRandomGenerator):
    """Allows one to obtain random values from a high-quality Random Number Generator."""
