from __future__ import annotations

from prettyqt.qt.QtOpenGLWidgets import *  # noqa: F403

from .openglwidget import OpenGLWidget
from prettyqt.qt import QtOpenGLWidgets

QT_MODULE = QtOpenGLWidgets


__all__ = ["OpenGLWidget"]
