from __future__ import annotations

from prettyqt.qt.QtPdfWidgets import *  # noqa: F403

from .pdfview import PdfView
from prettyqt.qt import QtPdfWidgets

QT_MODULE = QtPdfWidgets

__all__ = ["PdfView"]
